@@ -25,7 +25,7 @@
 #'     a character string e.g. 'tcp://127.0.0.1:5555' (see \link{transports}).
 #' @param auth [default NULL] an R object (possessed by both parties) which
 #'     serves as a pre-shared key on which to authenticate the communication.
-#'     Note: the object is never sent, only a random subset of its SHA256 hash.
+#'     Note: the object is never sent, only a random subset of its SHA-512 hash.
 #'
 #' @return Invisible NULL.
 #'
@@ -49,23 +49,20 @@
 #'
 messenger <- function(url, auth = NULL) {
 
-  nano_init(warn = "none")
-  on.exit(expr = {
-    options(warn = getOption("nanonext.original.warn"))
-    options(nanonext.original.warn = NULL)
-    invisible()
-  })
-  lock <- sha256(auth)
-  comb <- order(random(24L))
+  lock <- sha512(auth, convert = FALSE)
+  comb <- order(random(32L))
   key <- c(comb, as.integer(lock)[comb])
 
   sock <- .Call(rnng_messenger, url)
   is.integer(sock) && return(invisible(sock))
-  on.exit(expr = {
-    .Call(rnng_send, sock, writeBin(":d ", raw()), 0L, FALSE)
-    .Call(rnng_close, sock)
-  }, add = TRUE, after = FALSE)
 
+  nano_init(warn = "none")
+  on.exit(expr = {
+    send(sock, data = writeBin(":d ", raw()), mode = 2L, block = FALSE)
+    .Call(rnng_close, sock)
+    options(warn = getOption("nanonext.original.warn"), nanonext.original.warn = NULL)
+    invisible()
+  })
   cat("\n", file = stdout())
   intro <- unlist(strsplit("nanonext messenger", ""))
   for (i in seq_along(intro)) {
@@ -75,14 +72,14 @@ messenger <- function(url, auth = NULL) {
   cat(sprintf("\n| url: %s\n", url), file = stdout())
   cat("| connecting... ", file = stderr())
 
-  s <- .Call(rnng_send, sock, writeBin(":c ", raw()), 1000L, TRUE)
-  if (is.integer(s)) {
+  s <- send(sock, data = writeBin(":c ", raw()), mode = 2L, block = 1000L)
+  if (s) {
     cat(sprintf("\r| peer offline: %s\n", format.POSIXct(Sys.time())), file = stderr())
   } else {
     cat(sprintf("\r| peer online: %s\n", format.POSIXct(Sys.time())), file = stderr())
-    r <- .Call(rnng_recv, sock, 5L, TRUE, FALSE)
-    for (i in seq_len(24L)) {
-      lock[r[i]] == r[i + 24L] || {
+    r <- recv(sock, mode = 5L, block = TRUE)
+    for (i in seq_len(32L)) {
+      lock[r[i]] == r[i + 32L] || {
         cat("| authentication error\n", file = stderr())
         return(invisible())
       }
@@ -90,15 +87,15 @@ messenger <- function(url, auth = NULL) {
     cat("| authenticated\n", file = stderr())
   }
 
-  sock <- .Call(rnng_thread_create, list(sock, key))
+  sock <- .Call(rnng_thread_create, list(sock, key, 2L))
   cat("type your message:\n", file = stdout())
 
   repeat {
     data <- readline()
     if (identical(data, ":q")) break
     if (identical(data, "")) next
-    s <- .Call(rnng_send, sock, data, 0L, TRUE)
-    if (is.integer(s)) {
+    s <- send(sock, data = data, mode = 2L, block = FALSE)
+    if (s) {
       cat(sprintf("%*s > not sent: peer offline: %s\n", nchar(data), "", format.POSIXct(Sys.time())),
           file = stderr())
     } else {
