@@ -19,12 +19,11 @@
 #define NANONEXT_INTERNALS
 #define NANONEXT_PROTOCOLS
 #define NANONEXT_SUPPLEMENTALS
-#define NANONEXT_TIME
 #include "nanonext.h"
 
 // finalizers ------------------------------------------------------------------
 
-void stream_dialer_finalizer(SEXP xptr) {
+static void stream_dialer_finalizer(SEXP xptr) {
 
   if (R_ExternalPtrAddr(xptr) == NULL)
     return;
@@ -34,7 +33,7 @@ void stream_dialer_finalizer(SEXP xptr) {
 
 }
 
-void stream_listener_finalizer(SEXP xptr) {
+static void stream_listener_finalizer(SEXP xptr) {
 
   if (R_ExternalPtrAddr(xptr) == NULL)
     return;
@@ -44,7 +43,7 @@ void stream_listener_finalizer(SEXP xptr) {
 
 }
 
-void stream_finalizer(SEXP xptr) {
+static void stream_finalizer(SEXP xptr) {
 
   if (R_ExternalPtrAddr(xptr) == NULL)
     return;
@@ -63,17 +62,9 @@ SEXP rnng_strerror(SEXP error) {
 
 }
 
-SEXP rnng_scm(void) {
-  return R_MissingArg;
-}
-
-SEXP rnng_scm2(void) {
-  return R_UnboundValue;
-}
-
 SEXP rnng_clock(void) {
 
-  double time = (double) nng_clock();
+  const double time = (double) nng_clock();
   return Rf_ScalarReal(time);
 
 }
@@ -107,22 +98,73 @@ SEXP rnng_random(SEXP n) {
     vlen = (R_xlen_t) Rf_asInteger(n);
     break;
   default:
-    error_return("'n' must be integer or coercible to integer");
+    Rf_error("'n' must be integer or coercible to integer");
   }
 
-  vec = Rf_allocVector(REALSXP, vlen);
+  PROTECT(vec = Rf_allocVector(REALSXP, vlen));
   double *pvec = REAL(vec);
   for (R_xlen_t i = 0; i < vlen; i++) {
     pvec[i] = (double) nng_random();
   }
+
+  UNPROTECT(1);
   return vec;
 
 }
 
+SEXP rnng_device(SEXP s1, SEXP s2) {
+
+  if (R_ExternalPtrTag(s1) != nano_SocketSymbol)
+    Rf_error("'s1' is not a valid Socket");
+  if (R_ExternalPtrTag(s2) != nano_SocketSymbol)
+    Rf_error("'s2' is not a valid Socket");
+
+  const int xc = nng_device(*(nng_socket *) R_ExternalPtrAddr(s1),
+                            *(nng_socket *) R_ExternalPtrAddr(s2));
+  if (xc)
+    return mk_error(xc);
+
+  return nano_success;
+
+}
+
+// nano_init -------------------------------------------------------------------
+
+SEXP rnng_matchwarn(SEXP warn) {
+
+  if (TYPEOF(warn) == INTSXP) return warn;
+
+  const char *w = CHAR(STRING_ELT(warn, 0));
+  size_t slen = strlen(w);
+  const char i[] = "immediate", d[] = "deferred", e[] = "error", n[] = "none";
+  int xc = 0;
+
+  switch (slen) {
+  case 1:
+  case 2:
+  case 3:
+  case 4:
+    if (!strncmp(n, w, slen)) { xc = -1; break; }
+  case 5:
+    if (!strncmp(e, w, slen)) { xc = 2; break; }
+  case 6:
+  case 7:
+  case 8:
+    if (!strncmp(d, w, slen)) { xc = 0; break; }
+  case 9:
+    if (!strncmp(i, w, slen)) { xc = 1; break; }
+  default:
+      Rf_error("'warn' should be one of immediate, deferred, error, none");
+  }
+
+  return Rf_ScalarInteger(xc);
+
+}
+
 // ncurl - minimalist http client ----------------------------------------------
 
 SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP method, SEXP headers, SEXP data,
-                SEXP request, SEXP pem) {
+                SEXP response, SEXP pem) {
 
   nng_url *url;
   nng_http_client *client;
@@ -166,6 +208,7 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP method, SEXP headers, SEXP data,
     }
   }
   if (data != R_NilValue) {
+    data = nano_encode(data);
     unsigned char *dp = RAW(data);
     const size_t dlen = Rf_xlength(data) - 1;
     if ((xc = nng_http_req_set_data(req, dp, dlen)))
@@ -210,11 +253,13 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP method, SEXP headers, SEXP data,
   if (code != 200) {
     REprintf("HTTP Server Response: %d %s\n", code, nng_http_res_get_reason(res));
     if (code >= 300 && code < 400) {
-      SEXP ret = Rf_mkString(nng_http_res_get_header(res, "Location"));
+      SEXP ret;
+      PROTECT(ret = Rf_mkString(nng_http_res_get_header(res, "Location")));
       nng_http_res_free(res);
       nng_http_req_free(req);
       nng_http_client_free(client);
       nng_url_free(url);
+      UNPROTECT(1);
       return ret;
     }
   }
@@ -230,23 +275,22 @@ SEXP rnng_ncurl(SEXP http, SEXP convert, SEXP method, SEXP headers, SEXP data,
 
   SET_VECTOR_ELT(out, 0, Rf_ScalarInteger(code));
 
-  if (request != R_NilValue) {
-    const R_xlen_t rlen = Rf_xlength(request);
+  if (response != R_NilValue) {
+    const R_xlen_t rlen = Rf_xlength(response);
     PROTECT(rvec = Rf_allocVector(VECSXP, rlen));
-    SEXP rnames;
-
-    switch (TYPEOF(request)) {
+    switch (TYPEOF(response)) {
     case STRSXP:
       for (R_xlen_t i = 0; i < rlen; i++) {
-        const char *r = nng_http_res_get_header(res, CHAR(STRING_ELT(request, i)));
+        const char *r = nng_http_res_get_header(res, CHAR(STRING_ELT(response, i)));
         SET_VECTOR_ELT(rvec, i, r == NULL ? R_NilValue : Rf_mkString(r));
       }
-      Rf_namesgets(rvec, request);
+      Rf_namesgets(rvec, response);
       break;
-    case VECSXP:
+    case VECSXP: ;
+      SEXP rnames;
       PROTECT(rnames = Rf_allocVector(STRSXP, rlen));
       for (R_xlen_t i = 0; i < rlen; i++) {
-        SEXP rname = STRING_ELT(VECTOR_ELT(request, i), 0);
+        SEXP rname = STRING_ELT(VECTOR_ELT(response, i), 0);
         SET_STRING_ELT(rnames, i, rname);
         const char *r = nng_http_res_get_header(res, CHAR(rname));
         SET_VECTOR_ELT(rvec, i, r == NULL ? R_NilValue : Rf_mkString(r));
@@ -489,194 +533,15 @@ SEXP rnng_stream_listen(SEXP url, SEXP textframes, SEXP pem) {
 SEXP rnng_stream_close(SEXP stream) {
 
   if (R_ExternalPtrTag(stream) != nano_StreamSymbol)
-    error_return("'stream' is not a valid stream");
+    Rf_error("'stream' is not a valid stream");
   if (R_ExternalPtrAddr(stream) == NULL)
-    error_return("'stream' is not an active stream");
+    Rf_error("'stream' is not an active stream");
   nng_stream *sp = (nng_stream *) R_ExternalPtrAddr(stream);
   nng_stream_free(sp);
   R_ClearExternalPtr(stream);
   SET_ATTRIB(stream, R_NilValue);
 
-  return Rf_ScalarInteger(0);
-
-}
-
-// messenger -------------------------------------------------------------------
-
-static void thread_finalizer(SEXP xptr) {
-
-  if (R_ExternalPtrAddr(xptr) == NULL)
-    return;
-  nng_thread *xp = (nng_thread *) R_ExternalPtrAddr(xptr);
-  nng_thread_destroy(xp);
-  R_ClearExternalPtr(xptr);
-
-}
-
-static void rnng_thread(void *arg) {
-
-  SEXP list = (SEXP) arg;
-  SEXP socket = VECTOR_ELT(list, 0);
-  SEXP key = VECTOR_ELT(list, 1);
-  nng_socket *sock = (nng_socket *) R_ExternalPtrAddr(socket);
-  unsigned char *buf;
-  size_t sz;
-  time_t now;
-  struct tm *tms;
-  int xc;
-
-  while (1) {
-    xc = nng_recv(*sock, &buf, &sz, 1u);
-    time(&now);
-    tms = localtime(&now);
-
-    if (xc) {
-      REprintf("| messenger session ended: %d-%02d-%02d %02d:%02d:%02d\n",
-               tms->tm_year + 1900, tms->tm_mon + 1, tms->tm_mday,
-               tms->tm_hour, tms->tm_min, tms->tm_sec);
-      break;
-    }
-
-    if (!strncmp((char *) buf, ":", 1)) {
-      if (!strcmp((char *) buf, ":c ")) {
-        REprintf("| <- peer connected: %d-%02d-%02d %02d:%02d:%02d\n",
-                 tms->tm_year + 1900, tms->tm_mon + 1, tms->tm_mday,
-                 tms->tm_hour, tms->tm_min, tms->tm_sec);
-        nng_free(buf, sz);
-        rnng_send(socket, key, Rf_ScalarLogical(0), Rf_ScalarLogical(0));
-        continue;
-      }
-      if (!strcmp((char *) buf, ":d ")) {
-        REprintf("| -> peer disconnected: %d-%02d-%02d %02d:%02d:%02d\n",
-                 tms->tm_year + 1900, tms->tm_mon + 1, tms->tm_mday,
-                 tms->tm_hour, tms->tm_min, tms->tm_sec);
-        nng_free(buf, sz);
-        continue;
-      }
-    }
-
-    Rprintf("%s\n%*s< %d-%02d-%02d %02d:%02d:%02d\n",
-            (char *) buf, (int) sz, "",
-            tms->tm_year + 1900, tms->tm_mon + 1, tms->tm_mday,
-            tms->tm_hour, tms->tm_min, tms->tm_sec);
-    nng_free(buf, sz);
-
-  }
-
-}
-
-SEXP rnng_messenger(SEXP url) {
-
-  const char *up = CHAR(STRING_ELT(url, 0));
-  nng_socket *sock = R_Calloc(1, nng_socket);
-  void *dlp;
-  uint8_t dialer = 0;
-  int xc;
-  SEXP socket, con;
-
-  xc = nng_pair0_open(sock);
-  if (xc) {
-    R_Free(sock);
-    return mk_error(xc);
-  }
-  dlp = R_Calloc(1, nng_listener);
-  xc = nng_listen(*sock, up, dlp, 0);
-  if (xc == 10 || xc == 15) {
-    R_Free(dlp);
-    dlp = R_Calloc(1, nng_dialer);
-    xc = nng_dial(*sock, up, dlp, 2u);
-    if (xc) {
-      R_Free(dlp);
-      R_Free(sock);
-      return mk_error(xc);
-    }
-    dialer = 1;
-
-  } else if (xc) {
-    R_Free(dlp);
-    R_Free(sock);
-    return mk_error(xc);
-  }
-
-  PROTECT(socket = R_MakeExternalPtr(sock, nano_SocketSymbol, R_NilValue));
-  R_RegisterCFinalizerEx(socket, socket_finalizer, TRUE);
-
-  PROTECT(con = R_MakeExternalPtr(dlp, R_NilValue, R_NilValue));
-  if (dialer)
-    R_RegisterCFinalizerEx(con, dialer_finalizer, TRUE);
-  else
-    R_RegisterCFinalizerEx(con, listener_finalizer, TRUE);
-  R_MakeWeakRef(socket, con, R_NilValue, TRUE);
-
-  UNPROTECT(2);
-  return socket;
-
-}
-
-SEXP rnng_thread_create(SEXP list) {
-
-  SEXP socket = VECTOR_ELT(list, 0);
-  nng_thread *thr;
-  SEXP xptr;
-
-  nng_thread_create(&thr, rnng_thread, list);
-
-  PROTECT(xptr = R_MakeExternalPtr(thr, R_NilValue, R_NilValue));
-  R_RegisterCFinalizerEx(xptr, thread_finalizer, TRUE);
-  R_MakeWeakRef(socket, xptr, R_NilValue, TRUE);
-
-  UNPROTECT(1);
-  return socket;
-
-}
-
-// device ----------------------------------------------------------------------
-
-SEXP rnng_device(SEXP s1, SEXP s2) {
-
-  if (R_ExternalPtrTag(s1) != nano_SocketSymbol)
-    error_return("'s1' is not a valid Socket");
-  if (R_ExternalPtrTag(s2) != nano_SocketSymbol)
-    error_return("'s2' is not a valid Socket");
-
-  int xc = nng_device(*(nng_socket *) R_ExternalPtrAddr(s1),
-                      *(nng_socket *) R_ExternalPtrAddr(s2));
-  if (xc)
-    return mk_error(xc);
-  return Rf_ScalarInteger(xc);
-
-}
-
-// nano_init -------------------------------------------------------------------
-
-SEXP rnng_matchwarn(SEXP warn) {
-
-  if (TYPEOF(warn) == INTSXP) return warn;
-
-  const char *w = CHAR(STRING_ELT(warn, 0));
-  size_t slen = strlen(w);
-  const char i[] = "immediate", d[] = "deferred", e[] = "error", n[] = "none";
-  int xc = 0;
-
-  switch (slen) {
-  case 1:
-  case 2:
-  case 3:
-  case 4:
-    if (!strncmp(n, w, slen)) { xc = -1; break; }
-  case 5:
-    if (!strncmp(e, w, slen)) { xc = 2; break; }
-  case 6:
-  case 7:
-  case 8:
-    if (!strncmp(d, w, slen)) { xc = 0; break; }
-  case 9:
-    if (!strncmp(i, w, slen)) { xc = 1; break; }
-  default:
-      error_return("'warn' should be one of immediate, deferred, error, none");
-  }
-
-  return Rf_ScalarInteger(xc);
+  return nano_success;
 
 }
 
