@@ -37,18 +37,18 @@ nanotest(n1$dialer_setopt("size", "recv-size-max", 8192) == 0)
 nanotest(n1$dialer_start() == 0L)
 nanotest(n1$dialer[[1]]$state == "started")
 
-nanotestwn(n$send(data.frame(), block = FALSE, echo = FALSE))
-nanotestwn(n1$recv(block = FALSE))
-nanotestwn(n1$send("test", mode = "raw", block = 500, echo = FALSE))
-nanotestwn(n$recv("character", block = 500))
+nanotestwn(n$send(data.frame(), block = FALSE))
+nanotestwn(n1$recv(block = FALSE, keep.raw = TRUE))
+nanotestwn(n1$send("test", mode = "raw", block = 500))
+nanotestwn(n$recv("character", block = 500, keep.raw = TRUE))
 nanotestwn(n1$send(c(1L, 0L, 0L), mode = "r"))
-nanotestwn(n$recv("int", keep.raw = FALSE))
+nanotestwn(n$recv("int", keep.raw = TRUE))
 saio <- n$send_aio(data.frame(), timeout = 500)
 nanotest(identical(is_aio(saio), TRUE))
 nanotest(identical(is_nano(saio), FALSE))
 nanotestn(saio[["aio"]])
 nanotestn(saio$result)
-raio <- n1$recv_aio(timeout = 500)
+raio <- n1$recv_aio(timeout = 500, keep.raw = TRUE)
 raio$newfield <- "doesnotwork"
 nanotest(is.null(raio$newfield))
 nanotest(is_aio(raio) == TRUE)
@@ -60,7 +60,7 @@ saio$newfield <- "doesnotwork"
 nanotest(is.null(saio$newfield))
 nanotest(is_aio(saio) == TRUE)
 nanotest(is.logical(unresolved(saio)))
-msg <- n1$recv_aio(mode = "numer", timeout = 500, keep.raw = FALSE)
+msg <- n1$recv_aio(mode = "numer", timeout = 500)
 nanotest(is.null(stop_aio(msg)))
 nanotest(is.null(stop_aio(n)))
 nanotest(identical(call_aio(msg), msg))
@@ -114,7 +114,7 @@ nanotest(setopt(ctx, "ms", "send-timeout", 1000) == 0L)
 csaio <- send_aio(ctx, data.frame(), mode = "seria", timeout = 500)
 nanotestn(call_aio(csaio)$result)
 nanotestwn(recv_aio(ctx1, timeout = 500, keep.raw = FALSE))
-nanotestwn(send(ctx, "context test", mode ="raw", block = 300, echo = FALSE))
+nanotestwn(send(ctx, "context test", mode ="raw", block = 300))
 nanotestwn(recv(ctx1, mode = "ch", block = 300))
 nanotestwn(send(ctx, data.frame(), mode = "seri", block = 400))
 msg <- recv_aio(ctx1, mode = "ser", timeout = 400, keep.raw = FALSE)
@@ -122,7 +122,7 @@ nanotest(is.logical(unresolved(msg)))
 nanotestn(call_aio(msg)$data)
 nanotest(unresolved(msg) == FALSE)
 nanotestwn(send(ctx, c(TRUE, FALSE, TRUE), mode = "raw", block = 300))
-msg <- recv_aio(ctx1, mode = "l", timeout = 300)
+msg <- recv_aio(ctx1, mode = "l", timeout = 300, keep.raw = TRUE)
 nanotestn(msg$raw)
 nanotestwn(request(ctx, c(1+3i, 4+2i), recv_mode = "complex", timeout = 300))
 nanotestwn(reply(ctx1, execute = identity, send_mode = "ra", timeout = 300))
@@ -158,28 +158,15 @@ nanotest(close(resp) == 0L)
 
 nanotestwn(ncurl("https://www.r-project.org/"))
 nanotestwn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = c(`Content-Type` = "text/plain"), data = "test"))
-nanotestwn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), request = list("Date", "Server")))
-put1 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", request = c("date", "server"))
+nanotestwn(ncurl("http://httpbin.org/post", convert = FALSE, method = "POST", headers = list(`Content-Type` = "text/plain"), response = list("Date", "Server")))
+put1 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = c(Authorization = "Bearer token"), data = "test", response = c("date", "server"))
 nanotestn(put1$raw)
 put2 <- ncurl("http://httpbin.org/put", async = TRUE, method = "PUT", headers = list(Authorization = "Bearer token"))
 nanotestn(call_aio(put2)$data)
 nanotest(is.null(stop_aio(put2)))
 nanotestwn(stream(dial = "tcp://127.0.0.1:5555"))
-
-nanotest(is.character(ver <- nng_version()) && length(ver) == 2L)
-nanotest(is.character(nng_error(0L)))
-nanotest(is_nul_byte(as.raw(0L)) && !is_nul_byte(NULL))
-nanotest(!is_error_value(1L))
-nanotestwn(messenger("invalidURL"))
-nanotestn(tryCatch(.mirai_scm(), error = function(e) 0L))
-nanotest(is.double(mclock()))
-nanotest(is.null(msleep(1)))
-nanotest(is.null(msleep("a")))
-nanotest(is.double(random()))
-nanotest(length(sha256("test")) == 32L)
-nanotest(is.raw(sha256(c(1.1, 2.2), as.raw(1L))))
-nanotest(inherits(sha256(data.frame(), "KEY"), "nanoHash"))
-nanotest(length(as.character(sha256(as.raw(0L), key = NULL))) == 1L)
+nanotestw(stream(listen = "errorValue3") == 3L)
+nanotest(tryCatch(stream(), error = function(e) 0L) == 0L)
 
 nanotest(inherits(bus <- socket(protocol = "bus"), "nanoSocket"))
 nanotest(inherits(push <- socket(protocol = "push"), "nanoSocket"))
@@ -193,3 +180,28 @@ nanotest(close(pull) == 0L)
 nanotest(survey$close() == 0L)
 nanotest(respond$close() == 0L)
 
+nanotest(is.character(ver <- nng_version()) && length(ver) == 2L)
+nanotest(is.character(nng_error(0L)))
+nanotest(is_nul_byte(as.raw(0L)) && !is_nul_byte(NULL))
+nanotest(!is_error_value(1L))
+nanotestwn(messenger("invalidURL"))
+nanotest(is.double(mclock()))
+nanotest(is.null(msleep(1L)))
+nanotest(is.null(msleep(1)))
+nanotest(is.null(msleep("a")))
+nanotest(is.double(random()))
+nanotest(length(random(2)) == 2L)
+
+nanotest(sha256("test") == "9f86d081884c7d659a2feaa0c55ad015a3bf4f1b2b0b822cd15d6c15b0f00a08")
+nanotest(length(sha256(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 32L)
+nanotest(nchar(sha512(c(1.1, 2.2), as.raw(1L))) == 128L)
+nanotest(length(sha512("testing", convert = FALSE)) == 64L)
+nanotest(length(sha384(data.frame(), "KEY", convert = FALSE)) == 48L)
+nanotest(nchar(sha384(7000L, key = NULL, convert = TRUE)) == 96L)
+nanotest(length(sha224(c("one", "two"), key = "PRIVATE_key", convert = FALSE)) == 28L)
+nanotest(sha224(as.raw(0L)) == "fff9292b4201617bdc4d3053fce02734166a683d7d858a7f5f59b073")
+nanotest(base64enc("test") == "dGVzdA==")
+nanotest(base64dec(base64enc("test")) == "test")
+nanotest(is.raw(base64enc(data.frame(), convert = FALSE)))
+nanotest(is.integer(unserialize(base64dec(base64enc(c(1L, 2L)), convert = FALSE))))
+
